COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	Copyright (c) GeoWorks 1993 -- All Rights Reserved

PROJECT:	
MODULE:		
FILE:		powerUtils.asm

AUTHOR:		Adam de Boor, May 10, 1993

ROUTINES:
	Name			Description
	----			-----------

	
REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	Adam	5/10/93		Initial revision


DESCRIPTION:
	Utility routines that aren't logically a part of any particular
	facet of power management.
		

	$Id: powerUtils.asm,v 1.1.1.3 93/07/22 21:29:57 chrisb Exp $

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

if DISPLAY_MESSAGES

Resident	segment	resource

COMMENT @----------------------------------------------------------------------

FUNCTION:	CallRoutineInUI

DESCRIPTION:	Call a routine, but do it in the UI thread

CALLED BY:	INTERNAL

PASS:
	ax, bx, cx, dx, si, - data for routine
	dibp - offset of routine to call

RETURN:
	none

DESTROYED:
	none

REGISTER/STACK USAGE:

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/CAVEATS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	Tony	4/22/93		Initial version

------------------------------------------------------------------------------@
CallRoutineInUI	proc	far	uses ax, bx, dx, di, bp
	.enter

	push	di
	push	si
	push	dx
	push	cx
	push	bx
	push	ax
	pushdw	dibp
	mov	bp, sp				;ss:bp = parameters

	mov	ax, SGIT_UI_PROCESS
	call	SysGetInfo			;ax = ui handle
	mov_tr	bx, ax

	mov	ax, MSG_PROCESS_CALL_ROUTINE
	mov	dx, size ProcessCallRoutineParams
	mov	di, mask MF_STACK or mask MF_FORCE_QUEUE
	call	ObjMessage

	add	sp, dx

	.leave
	ret

CallRoutineInUI	endp

Resident	ends

;---

idata segment

;
; Queue to hold responses
;
messageResponseQueue	word	0
;
; Number of message dialogs displayed
;
messageCount		word	0

idata ends

Movable	segment	resource

COMMENT @----------------------------------------------------------------------

FUNCTION:	DisplayMessage

DESCRIPTION:	Display a message

CALLED BY:	INTERNAL

PASS:
	ax - CustomDialogBoxFlags
	si - chunk handle of message (in StringsUI resource)

RETURN:
	none

DESTROYED:
	none

REGISTER/STACK USAGE:

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/CAVEATS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	Tony	4/23/93		Initial version

------------------------------------------------------------------------------@
DisplayMessage	proc	far	uses ax, bx, cx, dx, si, di, bp, ds
params	local	GenAppDoDialogParams
	.enter

	; deal with the event queue

	push	ax, si
	mov	ax, dgroup
	mov	ds, ax
	mov	bx, ds:[messageResponseQueue]
	tst	bx
	jnz	queueExists

	; no event queue exists -- create one

	call	GeodeAllocQueue
	mov	ds:[messageResponseQueue], bx
	jmp	afterQueue

queueExists:

	; if there are any messages in the queue then delete them and reduce
	; the count

	call	GeodeInfoQueue
	tst	ax
	jz	afterQueue
	call	QueueGetMessage
	mov_tr	bx, ax
	call	ObjFreeMessage
	dec	ds:[messageCount]
afterQueue:
	pop	ax, si

	; if there is a message being displayed then bail

	tst	ds:[messageCount]
	jnz	done

	inc	ds:[messageCount]

	; do a warning

	mov	params.GADDP_dialog.SDP_customFlags, ax
	clr	bx
	clrdw	params.GADDP_dialog.SDP_stringArg1, bx
	clrdw	params.GADDP_dialog.SDP_stringArg2, bx
	clrdw	params.GADDP_dialog.SDP_customTriggers, bx
	clrdw	params.GADDP_dialog.SDP_helpContext, bx
	mov	bx, ds:[messageResponseQueue]
	mov	params.GADDP_finishOD.handle, bx
	mov	params.GADDP_message, MSG_META_DUMMY
	mov	bx, handle StringsUI
	call	MemLock
	mov	ds, ax
	mov	si, ds:[si]
	movdw	params.GADDP_dialog.SDP_customString, dssi

	clr	bx
	call	GeodeGetAppObject
	mov	ax, MSG_GEN_APPLICATION_DO_STANDARD_DIALOG
	mov	dx, size GenAppDoDialogParams
	mov	di, mask MF_CALL or mask MF_STACK
	push	bp
	lea	bp, params
	call	ObjMessage
	pop	bp

	mov	bx, handle StringsUI
	call	MemUnlock
done:
	.leave
	ret

DisplayMessage	endp

Movable	ends

endif
