COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

PROJECT:	PC GEOS
MODULE:		Com43 -- terminal emulation
FILE:		mouse.asm

AUTHOR:		Adam de Boor

ROUTINES:
	Name			Description
	----			-----------
	MouseStart		Starts using the mouse
	MouseEnd		Ends using the mouse
	MouseCheckCursor	Removes cursor if in the way
	MouseRestoreCursor	Put the cursor back if it was removed
	MouseRemoveCursor	Unconditionally remove the cursor

	
REVISION HISTORY:
	Name	Date	Description
	----	----	-----------
	ardeb	6/20/88	Initial Revision


DESCRIPTION:
	The functions in this file control the LogiTech mouse for com43.
	This allows me to use the mouse for emacs. ooo what fun. When a
	button is pressed, the string <meta-m><bstring><x><y> is sent as
	keyboard input. <bstring> is one of the constants defined in
	pc-mouse.el while <x> and <y> are the character positions (with a
	space added)

	Before a character is written to the screen, MouseCheckCursor should
	be called to remove the software cursor if it is in a conflicting
	position.
		
	$Id: mouse.asm,v 2.0 89/11/06 17:35:03 adam Exp $

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
MOUSEMODULE	= 1
		include	com43.def
program		segment

mousecall	macro	num
		;;
		;; Place the function number in ax. Note this is different
		;; from other systems that have the function in ah.
		;; 
		mov	ax, num
		int	51
		endm

MOUSEINIT	= 0		; Initialize mouse
MOUSESHOW	= 1		; Show cursor
MOUSEHIDE	= 2		; Hid cursor
MOUSESTAT	= 3		; Return status. BX = buttons,
				; CX = X (pixels) DX = Y (pixels)
MOUSEWARP	= 4		; Change mouse position. CX = X (pixels)
				; DX = Y (pixels)
MOUSELIMX	= 7		; Set X limits. CX = min (pixels),
				; DX = max (pixels)
MOUSELIMY	= 8		; Set Y limits. CX = min (pixels),
				; DX = max (pixels)
MOUSETEXT	= 10		; Define text cursor. BX = 1 for hardware or
				; 0 for software. CX = screen mask. DX =
				; cursor mask
MOUSEHANDLE	= 12		; Define handler. CX = call mask. ES:DX is
				; handler address.
MOUSEACCEL	= 30		; Set mouse accelerator. BX = type,
				; ES:DX = vector address
MOUSEMODE	= 33		; Set mouse mode. BX = 0 for read mode,
				; 1 for keyboard emulation mode and 2 for
				; inactive
;
; Bit masks for event type. These masks are OR'ed together to reflect
; changes since the last time the event handler was called.
;
MOUSEMOTION	=	00000001b	; Mouse moved
MOUSELPRESS	=	00000010b	; Left button pressed
MOUSELREL	=	00000100b	; Left button released
MOUSERPRESS	=	00001000b	; Right button pressed
MOUSERREL	=	00010000b	; Right button released
MOUSEMPRESS	=	00100000b	; Middle button pressed
MOUSEMREL	=	01000000b	; Middle button released
MOUSEALL	=	01111111b	; All events

mouseX		db	0	; Current mouse X coordinate
mouseY		db	0	; Current mouse Y coordinate
mouseFlags	db	0
MOUSEON		=	00000001b
MOUSECURSIN	=	00000010b
		bugsym	mouseX
		bugsym	mouseY
		bugsym	mouseFlags

BPRESS		STRUC
button		db	0	; Button code
xcoord		db	0	; Previous X
ycoord		db	0	; Previous Y
BPRESS		ENDS

mouseLast	BPRESS	<>
mouseAcked	db	0	; Non-zero if can repeat press on motion

;
; Mouse accelerator. Perhaps this should be user-settable? Need to set it
; b/c those people that use mouse.com get a wild default accelerator.
; This table represents an acceleration of 2 for movements over 10
;
mouseAcc	db	0, 1, 2, 3, 4, 5, 6, 7
		db	8, 9, 10, 22, 24, 26, 28, 30
		db	32, 34, 36, 38, 40, 42, 44, 46
		db	48, 50, 52, 54, 56, 58, 60, 62
		db	64, 66, 68, 70, 72, 74, 76, 78
		db	80, 82, 84, 86, 88, 90, 92, 94
		db	96, 98, 100, 102, 104, 106, 108, 110
		db	112, 114, 116, 118, 120, 122, 124, 126
		db	128, 130, 132, 134, 136, 138, 140, 142
		db	144, 146, 148, 150, 152, 154, 156, 158
		db	160, 162, 164, 166, 168, 170, 172, 174
		db	176, 178, 180, 182, 184, 186, 188, 190
		db	192, 194, 196, 198, 200, 202, 204, 206
		db	208, 210, 212, 214, 216, 218, 220, 222
		db	224, 226, 228, 230, 232, 234, 236, 238
		db	240, 242, 244, 246, 248, 250, 252, 254


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		MouseStart
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Turn on the mouse

CALLED BY:	KbdHandleSys

PASS:		Nothing

RETURN:		Nothing

DESTROYED:

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/20/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		public	MouseStart
MouseStart	proc	near
		push	ax
		push	bx
		push	cx
		push	dx

		;
		; See if mouse attached. If AX returns 0, there's no mouse.
		;
		mousecall MOUSEINIT

		comment @
		; logitech mouse driver restores all registers on return
		; from function 0, so this test won't work
		tst	ax
		jz	MouseStartRet
		int	3
		@
		mov	mouseFlags, MOUSEON

		;
		; Turn on the mouse (in mouse mode)
		;
		;mov	bx, 0
		;mousecall MOUSEMODE

		;
		; First set the text cursor. Preserve the foreground, but
		; set the background to be the mouse's assigned color: 4
		;
		mov	bx, 0		; software cursor
		mov	cx, 0000111111111111b
		mov	dx, 0100000000000000b
		test	scrEtcFlags, SCR_ISMONO
		jz	MS1
		;
		; Monochrome: nuke high bits on both foreground and background
		; attributes, and invert the low three bits of each. Should
		; result in inversion...
		;
		mov	cx, 0111011111111111b
		mov	dx, 0111011100000000b
MS1:
		mousecall MOUSETEXT

		;
		; Set up our event handler
		;
		mov	cx, MOUSEALL
		mov	ax, cs
		mov	es, ax
		mov	dx, offset MouseEvent
		mousecall MOUSEHANDLE

		;
		; Set cursor boundaries. X boundaries already set.
		;
		mov	dx, scrLines	; Fetch # of lines in screen
		sal	dx, 1		; *8 to get pixels
		sal	dx, 1
		sal	dx, 1
		dec	dx		; -1 since it's 0-origin...
		mov	cx, 0		; Min Y
		mousecall MOUSELIMY

		;
		; Install accelerator
		;
		;mov	dx, offset mouseAcc
		;mov	bx, 1		; Use given accelerator
		;mousecall MOUSEACCEL

		clr	cx
		clr	dx
		mousecall MOUSEWARP

		;
		; Find where the thing is currently
		;
		mousecall MOUSESTAT
		shr	cx, 1		; convert X pixels to chars
		shr	cx, 1
		shr	cx, 1
		mov	mouseX, cl
		shr	dx, 1		; convert Y pixels to chars
		shr	dx, 1
		shr	dx, 1
		mov	mouseY, dl

		mousecall MOUSESHOW

MouseStartRet:
		pop	dx
		pop	cx
		pop	bx
		pop	ax
		ret
MouseStart	endp



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		MouseEnd
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Turn off the mouse again

CALLED BY:	KbdHandleSys

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/20/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		public	MouseEnd
MouseEnd	proc	near
		push	ax
		push	bx
		push	cx

		mousecall MOUSEHIDE

		;
		; Turn off the event handler
		;
		mov	cx, 0		; Nothing
		mov	dx, offset MouseEvent
		mousecall MOUSEHANDLE

		mov	mouseFlags, 0
		mov	bx, 2
		mousecall MOUSEMODE

		pop	cx
		pop	bx
		pop	ax
		ret
MouseEnd	endp


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		MouseCheckCursor
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Remove the cursor if it's in the way

CALLED BY:	ScrWrite

PASS:		CX	= X position
		BX	= Y position

RETURN:		Nothing

DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/20/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		public	MouseCheckCursor
MouseCheckCursor proc	near
		cmp	mouseFlags, MOUSEON OR MOUSECURSIN
		jne	MCRet

		cmp	cl, mouseX
		jne	MCRet
		cmp	bl, mouseY
		jne	MCRet

		;
		; Remove the cursor
		;
		push	ax
		mousecall MOUSEHIDE
		pop	ax
		and	mouseFlags, NOT MOUSECURSIN
MCRet:
		ret
MouseCheckCursor endp


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		MouseRemoveCursor
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Unconditionally remove the cursor if it's there

CALLED BY:	Scr*

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/20/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		public	MouseRemoveCursor
MouseRemoveCursor proc	near
		push	ax
		cmp	mouseFlags, MOUSEON OR MOUSECURSIN
		jne	MRemCRet
		mousecall MOUSEHIDE
		and	mouseFlags, NOT MOUSECURSIN
MRemCRet:
		pop	ax
		ret
MouseRemoveCursor endp



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		MouseRestoreCursor
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Restore the software cursor if it was removed

CALLED BY:	Main

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	AX

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/20/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		public	MouseRestoreCursor
MouseRestoreCursor proc	near
		test	mouseFlags, MOUSEON
		jz	MRCRet
		test	mouseFlags, MOUSECURSIN
		jnz	MRCRet
		test	scrEtcFlags, SCR_ISSAVED	; Don't restore it if
							; the screen is off
		jnz	MRCRet
		mousecall MOUSESHOW
		or	mouseFlags, MOUSECURSIN
MRCRet:
		ret
MouseRestoreCursor endp



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		MouseStuff
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Stuff a mouse string into the input stream

CALLED BY:	MouseEvent

PASS:		AL	= modifier mask
		BL	= button mask
		DS	= our segment

RETURN:		Nothing

DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/20/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
LPRESS		= 010b
LREL		= 110b
MPRESS		= 001b
MREL		= 101b
RPRESS		= 000b
RREL		= 100b
mouseStuffStr	db	'm' OR 80h	; meta-m
mouseButton	db	0
mouseXPos	db	0
mouseYPos	db	0

MouseStuff	proc	near
		push	ax
		push	cx
		push	si

		or	al, bl
		mov	mouseButton, al
		test	al, 100b
		jnz	MS2		; It's a button release, so
					; don't save it.
		mov	mouseLast.button, al
		mov	al, mouseX
		mov	mouseLast.xcoord, al
		mov	al, mouseY
		mov	mouseLast.ycoord, al
MS2:
		mov	al, mouseX
		add	al, ' '
		mov	mouseXPos, al
		mov	al, mouseY
		add	al, ' '
		mov	mouseYPos, al
		mov	si, offset mouseStuffStr
		mov	cx, 4
		call	KbdInsertString

		mov	mouseAcked, 0
		pop	si
		pop	cx
		pop	ax
		ret
MouseStuff	endp
		bugsym	MouseStuff


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		MouseEvent
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Handle a mouse event

CALLED BY:	Mouse driver

PASS:		CS	= our code segment
		AX	= mask of event type (see beginning of file)
		BX	= current button state:
				bit 0 = left button
				bit 1 = right button
				bit 2 = middle button
		CX	= current X position (in pixels)
		DX	= current Y position (in pixels)

RETURN:		Nothing

DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:
	If the event contains mouse motion, update our idea of where the
	cursor is. The driver takes care of the cursor.

	If it's a mouse button, we need to shove a string into the
	input stream.\M-m<button><x><y>

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/20/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		bugsym	MouseEvent
MouseEvent	proc	far
		push	ds
		push	cs
		pop	ds		; Set ds for convenience

		call	ScrEnable	; Tell screen to turn on.

ifdef MOUSE_DEBUG
		push	ax
		call	ScrPrintByte
		pop	ax
endif
		test	al, MOUSEMOTION
		jz	ME2
		;
		; The characters are in an 8x8 box, so divide both coordinates
		; by 8 to get the character position.
		;
		shr	cx, 1
		shr	cx, 1
		shr	cx, 1
		mov	mouseX, cl
		shr	dx, 1
		shr	dx, 1
		shr	dx, 1
		mov	mouseY, dl
		;
		; Make sure the buttons didn't change
		;
		test	al, NOT MOUSEMOTION
		jnz	ME2		; If buttons changed, handle them
		;
		; See if any buttons down and repeat the last press at the
		; new position if so
		;
		tst	bx
		jnz	MEButtonMotion
ME2:
		;
		; Fetch current modifiers and leave them in the high nibble of
		; al.
		;
		mov	cx, ax
		call	KbdGetMods
		shl	ax, 1
		shl	ax, 1
		shl	ax, 1
		shl	ax, 1

		;
		; Check left button. Note that we can't have both press and
		; release of the same button...I think.
		;
		test	cl, MOUSELPRESS
		jz	ME3
		mov	bl, LPRESS
		call	MouseStuff
		jmp	short ME4
ME3:		test	cl, MOUSELREL
		jz	ME4
		mov	bl, LREL
		call	MouseStuff

ME4:
		;
		; Check middle button
		;
		test	cl, MOUSEMPRESS
		jz	ME5
		mov	bl, MPRESS
		call	MouseStuff
		jmp	short ME6
ME5:		test	cl, MOUSEMREL
		jz	ME6
		mov	bl, MREL
		call	MouseStuff

ME6:
		;
		; Check right button
		;
		test	cl, MOUSERPRESS
		jz	ME7
		mov	bl, RPRESS
		call	MouseStuff
		jmp	short ME8
ME7:
		test	cl, MOUSERREL
		jz	ME8
		mov	bl, RREL
		call	MouseStuff
ME8:
MERet:
		pop	ds
		ret
MEButtonMotion:
		;
		; Repeat the last button-press command
		;
		test	mouseAcked, 1
		jz	ME2		; Previous not acknowledged, so
					; can't repeat.
		cmp	mouseLast.ycoord, cl
		jne	MEB2
		cmp	mouseLast.xcoord, dl
		je	ME2		; Both the same -- no motion
MEB2:

		mov	mouseLast.ycoord, cl
		mov	mouseLast.xcoord, dl

		add	cl, ' '
		mov	mouseXPos, cl
		add	dl, ' '
		mov	mouseYPos, dl
		push	si
		push	ax
		mov	al, mouseLast.button
		mov	mouseButton, al
		mov	si, offset mouseStuffStr
		mov	cx, 4
		call	KbdInsertString
		pop	ax
		pop	si
		jmp	ME2
MouseEvent	endp




COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		MouseAcknowledge
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Note that the last button event was acknowledged

CALLED BY:	ScrParseSeq

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:
	Set the low bit of mouseAcked

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/24/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		public	MouseAcknowledge
MouseAcknowledge proc	near
		mov	mouseAcked, 1
		ret
MouseAcknowledge endp



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		MouseWarp
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Set the position of the mouse cursor

CALLED BY:	ScrWrite upon reading both args to the WM sequence
PASS:		CX	= new X position
		DX	= new Y position
RETURN:		Nothing
DESTROYED:	AX

PSEUDO CODE/STRATEGY:
	Since we are passed the args in the registers in which the driver
	expects its args, we have only to load AX, convert them to pixels
	and trap.

KNOWN BUGS/SIDE EFFECTS/IDEAS:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	9/30/88		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		bugsym	MouseWarp
MouseWarp	proc	near
		mov	mouseX, cl	; Record new position
		mov	mouseY, dl

		sal	cx, 1		; The driver wants it in pixels.
		sal	cx, 1		; Since the characters are 8x8,
		sal	cx, 1		; just multiply both coords by 8
		sal	dx, 1		; to get the pixel position.
		sal	dx, 1
		sal	dx, 1
		
		mousecall MOUSEWARP
		ret
MouseWarp	endp
program		ends
		end
