COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

PROJECT:        PC GEOS
MODULE:         Com43 -- Communications program for MS-DOS entry and exit
FILE:           main.asm

AUTHOR:         Adam de Boor

ROUTINES:
	Name            Description
	----            -----------
	Main            Entry point
	Exit            Exit point
	SetInterrupt    Set an interrupt vector to one of our own routines
	ResetInterrupt  Restore an interrupt vector to what it used to be.

	
REVISION HISTORY:
	Name    Date    Description
	----    ----    -----------
	adam    6/7     Initial Revision


DESCRIPTION:
	This program is used to communicate between a PC and another
	system -- in this case UNIX. It performs most simple terminal
	functions as well as file transfer. The file transfer is performed
	using a very simple protocol.

	A termcap entry is located in com43.ter.
		
	$Id: pccom.asm,v 1.5 93/06/04 17:30:03 jimmy Exp $
	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
MAINMODULE      =       1
		include com43.def

.ioenable

CMD_TAIL        = 80h           ; start of command tail
ENV_SEG         = 2ch           ; segment of the environment

program         segment 

		org     100h    ; Must start at 100h


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		Main
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:       Entry point for Com43

CALLED BY:      MS-DOS

PASS:           Nothing

RETURN:         Nothing

DESTROYED:      Everything

PSEUDO CODE/STRATEGY:
	Calls appropriate initialization routines then goes into an infinite
	loop checking the the keyboard and the serial line, in that order, for
	characters that need to be transfered from the one to the other.

	Return from this function occurs when exit is called, at which point
	things are cleaned up and the appropriate trap back into MS-DOS is
	performed.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name    Date            Description
	----    ----            -----------
	adam    6/7/88          Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

Main            proc    far     
if DEBUG and not DEBUG_USE_BIOS
	; clear the screen and set the initial blinking cursor
		les     di, cs:[curPos]
		mov     cx, SCREEN_SIZE/2
		mov     ax, (SCREEN_ATTR_NORMAL shl 8) or ' '
		rep     stosw
		mov     di, cs:[curPos].offset
		mov     ax, (SCREEN_ATTR_NORMAL or 0x80) or 0x7f
		stosw
		push    ds
		pop     es
endif
		mov     ds:[pc_unix], PCCOM
		call    CheckIsPC       ; check for running on a pc.
					; Sets 'isBasePC'.
		call    ScrInit
		call    ScrWriteUserString
		call    ComInit
		call    KbdInit

		test    ds:[sysFlags], MASK SF_SWAT
		je      determineVersion

		;
		; Just got back from running swat, so tell pcwin to redraw.
		; If you weren't running pcwin while you were running swat,
		; you deserve this escape sequence.
		;
		mov     si, offset pcwinReset
		mov     cx, 4
		call    ComWriteBlock
Main2:
		and     ds:[sysFlags], NOT MASK SF_SWAT
		;
		; Check for /m arg to turn on the mouse
		;
		mov     di, offset mouseArg
		clr     bx
		call    FetchArg
		tst     bx
		jz      main_loop       ; no /m arg
		call    MouseStart      ; turn on mouse
main_loop:
		test    ds:[sysFlags], MASK SF_EXIT  ; Perform synchronous exit
		jnz     Exit
		test    ds:[sysFlags], MASK SF_SWAT
		jnz     RunSwat
		test    ds:[sysFlags], MASK SF_PRINT_SCREEN
		jz      no_print
		and     ds:[sysFlags], NOT MASK SF_PRINT_SCREEN
		int     5               ; Invoke printscreen interrupt
no_print:
		call    MouseRestoreCursor
		call    KbdRead         ; read keystroke, if any
		jz      short no_key
		call    CheckForExitKey
		jc      Exit
		call    ComWrite        ; write it to the serial line
no_key:
		call    ComRead         ; read serial line, if any character
					; available
		jz      main_loop
		call    ScrWrite
		jmp     main_loop

Exit            label   near
		call    ScrExitToDos
		.unreached

determineVersion:
		mov     dx, offset swatRun1_2EC+1
		mov     ax, 0x4300      ; get attributes
		int     21h
		jnc     Main2           ; version 1.2 -- leave as is
		
		mov     dx, offset swatRun1_2NonEC+1
		mov     ax, 0x4300
		int     21h
		jnc     Main2           ; version 1.2 -- leave as is
		
		mov     ds:[swatArgPtrs], offset swatArg2Ptrs
		mov     ds:[swatArgs], offset swatRun2EC

		mov     dx, offset swatRun2EC+1
		mov     ax, 0x4300      ; get attributes
		int     21h
		jnc     Main2           ; version 2 -- leave as changed
		
		mov     dx, offset swatRun2NonEC+1
		mov     ax, 0x4300
		int     21h
		jnc     Main2           ; version 2 -- leave as changed
		
		; version 1.0 or 1.1
		mov     ds:[swatArgPtrs], offset swatArg1Ptrs
		mov     ds:[swatArgs], offset swatRun1EC
		jmp     Main2
Main            endp


program         ends
		end     Main
