COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	Copyright (c) GeoWorks 1991 -- All Rights Reserved

PROJECT:	PC GEOS
MODULE:		
FILE:		pcsend.asm

AUTHOR:		jimmy lefkowitz

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	7/31/92		Initial version.

DESCRIPTION:
	Code for sending a file via the serial port.

	The file transfer is performed using the following protocol:
	(See complete documentation in Tools section 10.8.3)

	sender: <Esc>XF 1 filename\0	     
	reciever: sync
	sender: file size
< optional: 
	sender: BLOCK_START "!PCCom File Transfer Filename Block! " filename\0
		BLOCK_END CRC+1
>
loop:
	sender: BLOCK_START block_data BLOCK_END CRC
	reciever: sync
	loop until all blocks sent
	sender: checksum (0)
	reciever: ack


	If a block is not successfully sent, it is tried again up to three 
	times, at which point we give up.
 
	all multi-byte values are send low byte to high byte
	checksum is a 4 byte quantity	
	packets are any length, up to 1K
	sync is ffh
	ack = 0 for ok, 1 for not ok


	$Id: pcsend.asm,v 1.6 94/01/22 14:45:13 cassie Exp $

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

include com43.def

program segment
	

COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		Main
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Entry point for pcsend

CALLED BY:	MS-DOS

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	Everything

PSEUDO CODE/STRATEGY:
	Calls appropriate initialization routines then goes into an infinite
	loop checking the the keyboard and the serial line, in that order, for
	characters that need to be transfered from the one to the other.

	Return from this function occurs when exit is called, at which point
	things are cleaned up and the appropriate trap back into MS-DOS is
	performed.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		org	100h	; Must start at 100h

Main		proc 	far	
		.enter
		call	MainCommon
		jc	done
sendLoop:
		call	GetNextFileName	
		jc	done
		call	SendFile
		jmp	sendLoop	
done:
		.leave
		call	ComExit

		mov	al, 0		; exit(0)
		mov	ah, DOS_EXIT
		int	21h
		.unreached
		ret
Main		endp



	

fileflag	byte	'f',0
fileHandle	word	?
buf		byte	BUFFER_SIZE		dup(?)
tries		byte	0



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		SendFile
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	send a file through the serial port

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	7/29/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
SendFile	proc	near
	uses	ds, ax, dx
	.enter

	; open the dos file first too make sure its really there
	mov	ah, DOS_FILE_OPEN
	mov	al, 0		; file attributes (read only, no access denied)
	mov	dx, offset filename
	push	cs
	pop	ds
	int	21h		; ax <- file handle
LONG	jc	fileOpenError	
	mov	cs:[fileHandle], ax	

	; now send out an esacpe sequence telling com43 that a file
	; is coming

	mov	al, 1bh		; esacpe code
	call	ComWrite
	mov	al, 'X'
	call	ComWrite
	mov	al, 'F'
	call	ComWrite
	mov	al, RECV_COMMAND	; recieve a file
	call	ComWrite

	; sync up before sending filename
	; 

	lea	bx, filename
	mov	ah, SCREEN_ATTR_NORMAL
	mov	si, offset directory
	cmp	{byte}cs:[si], 0
	jz	nameLoop	
directoryLoop:
	mov	al, cs:[si]
	tst	al
	jnz	writeDirByte
	; write out a \ in place of the null terminator...
	mov	al, '\\'
	call	ComWrite
	jmp	nameLoop
writeDirByte:
	call	ComWrite
	inc	si
	jnz	directoryLoop	
nameLoop:
	mov	al, cs:[bx]
	push	ax
	call	ComWrite		
	pop	ax
	inc	bx
	tst	al
	jnz	nameLoop

	; we must wait for a sync, this will get handled by the interrupt
	; handler, which takes any incoming bytes off the serial port
	; and puts em into the read buffer, so ComRead will find them
	; and we can be on our way...
	call	Sync
LONG	jc	abort
	cmp	al, NAK and 07fh
LONG	je	abort
	cmp	al, NAK_QUIT and 07fh
LONG	je	abort
	; I will get the file size by setting the file pointer to the end
	; of the file and seeing where it lands up
	mov	ah, DOS_FILE_SEEK
	mov	al, SEEK_END
	mov	bx, cs:[fileHandle]
	mov	cx, 0		; offset send end = 0 
	mov	dx, cx	
	int	21h		; dx:ax = new file position

if	DEBUG
	call	DebugPrintWord
	push	ax
	mov	ax, dx
	call	DebugPrintWord
	pop	ax
endif

	; now send out the file size in appropriate byte order
	call	ComWrite
	mov	al, ah
	call	ComWrite
	mov	al, dl
	call	ComWrite
	mov	al, dh
	call	ComWrite

	; now reset the file pointer to the beginning
	mov	ah, DOS_FILE_SEEK
	mov	al, SEEK_START
	mov	cx, 0
	mov	dx, cx
	mov	bx, cs:[fileHandle]
	int	21h

	call	SendFilenameBlock
	jc	abort				; receiver sent NAK_QUIT

readLoop:
	mov	dx, offset buf
	mov	cx, BUFFER_SIZE	; number of bytes to read
	mov	bx, cs:[fileHandle]
	mov	ah, DOS_FILE_READ
	int	21h		; DOS FILE READ
	jc	fileReadError
	xchg	ax, cx
	; if the file size is exactly divisible by BUFFER_SIZE
	; cx will be zero after we have read the last block
	jcxz	sendChecksum
	mov	tries, 0

sendBlock:
	mov	si, offset buf
	call	WriteDataBlock	
	call	Sync

	jc	reSend
	cmp	al, NAK
	je	reSend
	cmp	al, NAK_QUIT
	je	abort
	jmp	readLoop

reSend:
	; only retry so many times....
	cmp	cs:[tries], MAX_TRIES
	jge	abort
	inc	cs:[tries]
	jmp	sendBlock

	; now send the checksum, which is just 0.
	;
sendChecksum:
	clr	al
	call	ComWrite
	call	ComWrite

	; now we must make sure that the stuff we have sent out to the
	; port via ComWrite actually gets sent out, as it might have been
	; queued up and if we just exit, the stuff still waiting to be sent
	; from the queue gets screwed, and we wouldn't want that
	call	ComDrainQueue
done:		
	mov	bx, cs:[fileHandle]
	tst	bx
	jz	Exit
	mov	ah, DOS_FILE_CLOSE
	int	21h		; DOS FILE CLOSE
Exit:
	.leave
	ret
fileOpenError:
	call	FileOpenError
	jmp	Exit
fileReadError:
	call	FileReadError
	jmp	done
abort:
	call	Abort
	jmp	done
SendFile	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		SendFilenameBlock
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Send the filename in the first block.

CALLED BY:	SendFile
PASS:		nothing
RETURN:		carry set if receiver sent NAK_QUIT
DESTROYED:	

PSEUDO CODE/STRATEGY:
KNOWN BUGS/SIDE EFFECTS/IDEAS:
REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	cassie	12/29/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
SendFilenameBlock		proc	near
		.enter

		mov	ax, cs
		mov	es, ax
		mov	ds, ax
		mov	di, offset buf	; es:di destination buffer for filename
		mov	cs:[tries], 0
		mov	ah, SCREEN_ATTR_NORMAL

	;
	; Copy the filenameBlockString to the buffer
	;
		mov	si, offset filenameBlockString
		mov	cx, offset filenameBlockStringEnd
		sub	cx, si
		mov	bx, cx
		rep	movsb

	;
	; Copy the directory path and filename into a buffer
	;
		clr	cx		; filename length
		mov	si, offset filename
		cmp	ds:[directory], 0
		jz	nameLoop	
		mov	si, offset directory
directoryLoop:
		lodsb	
		tst	al
		jnz	writeDirByte
		mov	al, '\\' 	; write a `\\` instead of null-term.
		stosb
		inc	cx
		mov	si, offset filename
		jmp	nameLoop
writeDirByte:
		stosb	
		inc	cx
		jnz	directoryLoop	
nameLoop:
		lodsb	
		stosb
		inc	cx
		tst	al
		jnz	nameLoop

		add	bx, cx			; bx <- block length
	;
	; Now we have the block string and filename in a buffer, send it.
	;
sendBlock:
		mov	al, BLOCK_START		; send the block start byte
		call	ComWrite

		mov	si, offset buf		; ds:si <- filename
		mov	cx, bx
dataLoop:
		lodsb
		call	ComWrite
		loop	dataLoop

	; send the END token
		mov	al, BLOCK_END
		call	ComWrite

	; calculate the block CRC and add one
		clr	ax
		mov	cx, bx
		mov	di, offset buf
		call	CalcCRC
		inc	ax

	; send the block CRC
		call	ComWrite
		mov	al, ah
		call	ComWrite

	; sync up with receiver
		call	Sync

		jc	reSend			; timed out
		cmp	al, NAK			
		je	reSend
		cmp	al, NAK_QUIT
		stc
		je	exit
done:
		clc
exit:
		.leave
		ret

reSend:
	; Resend only MAX_TRIES-2 times, so that if receiver is using
	; the old protocol, it won't abort the file transfer because it 
	; thinks this is a funky block.
	; If not successful sending this block after MAX_TRIES - 2 times,
	; the next block we send will be a real data block. Note that at 
	; that point the receiver has only 2 tries left to read a valid block.
	;
		cmp	cs:[tries], MAX_TRIES - 2
		jae	done
		inc	cs:[tries]
		jmp	sendBlock

SendFilenameBlock		endp
	
program ends


