##############################################################################
#
#       Copyright (c) Berkeley Softworks 1988 -- All Rights Reserved
#
# PROJECT:      PC GEOS
# MODULE:       PMake System Library
# FILE:         tool.mk
# AUTHOR:       Adam de Boor, Jun 19, 1989
#
# TARGETS:
#       Name                    Description
#       ----                    -----------
#       <mach>                  Where <mach> is one of the known machines,
#                               creates the tool in the directory <mach>.md
#       depend<mach>            Creates dependencies for the <mach> version
#                               of the tool
#       all                     Creates the tool for all known machines
#       depend                  Generates dependencies for all known machines
#       install                 Installs the tool for all known machines
#       MAKETOOL                .USE rule to create a machine-dependent version
#                               of a tool -- you must still perform a
#                               transformation on $(MACHINES) to generate all
#                               the targets, but this rule will compile it
#                               for you...
#
# REVISION HISTORY:
#       Name    Date            Description
#       ----    ----            -----------
#       ardeb   6/19/89         Initial Revision
#
# DESCRIPTION:
#       Makefile for tools. Expected variables:
#
#           MACHINES        list of all possible machines
#           DEFTARGET       default target to make
#           NAME            name of final executable
#           TYPE            contains "library" if the tool is actually a
#                           library.
#           INSTFLAGS       Flags for the "install" program
#           DEST            Place in which to install executable/library.
#                           Defaults to /usr/public.<mach>
#           LIBS            Any libraries required for all machine
#                           architectures (the link uses the appropriate
#                           machine-dependent subdirectory for each directory
#                           in the .LIBS variable)
#
#       For each machine <mach>:
#
#           <mach>SRCS      list of sources for machine
#           <mach>OBJS      list of object files for <mach> (with <mach>.md/
#                           at the front of each...)
#           <mach>LIBS      list of machine-specific libraries
#
###############################################################################

.MAIN           : $(DEFTARGET)

#if     defined(INSTALL_DIR)
#
# Define search paths for the various types of files we might be compiling.
# This is conditional to allow non-geos tools to be compiled with this
# makefile.
#
.PATH.h         : . $(INSTALL_DIR) \
		  $(DEVEL_DIR)/Tools/include $(ROOT_DIR)/Tools/include
.PATH.c         : $(INSTALL_DIR)
.PATH.y         : $(INSTALL_DIR)
.PATH.l         : $(INSTALL_DIR)
#endif

#
# Specify gcc-include directory to deal with /usr/include.<mach> given by
# gcc -- there are certain things (like assert.h) that we want from the
# gcc include directory, not the standard one.
#
#.PATH.h         : /usr/public/lib/gcc-include


# The use of -I- is required to ensure that any header in a development
# directory is used rather than its corresponding installed version regardless
# of where the including file is located.
#
AFLAGS          += -m$(.TARGET:H:R) $(XAFLAGS)
YFLAGS          += $(XYFLAGS)
INSTFLAGS       += -b $(XINSTFLAGS)
#
# Given multiple things to install, we need to get the md directory for
# only the first. Easiest way to do this is with the "set" command in
# the shell to break the thing into words, then using only $1
#
DEST            ?= `set - $(.ALLSRC:H:S/.md//); echo /usr/public.$1`
#if defined(LIBS) && $(LIBS) == "-lkernel.lobj"
LIBS            = 
#else
LIBS            ?=
#endif

#
# Variables for extra command-line flags, in case there are none...
#
XCFLAGS         ?= 
XAFLAGS         ?= 
XYFLAGS         ?= 
XINSTFLAGS      ?= 

#if $(TYPE) != "library"

#
# Create an executable from the appropriate objects and libraries
#

$(MACHINES)     : ${.TARGET:S%$%.md/$(NAME)%}               .JOIN
${MACHINES:S%$%.md/$(NAME)%g}   : MAKETOOL \
		  ${.TARGET:H:R:S/^/\$(/:S%$%OBJS)%} \
		  $(LIBS) \
		  ${.TARGET:H:R:S/^/\$(/:S%$%LIBS)%}

#else

#
# Archive the object files instead, using the MAKELIB rule to handle the
# object file management and removal.
#

many_libraries  = you bet your sweet behind...
#include    <makelib.mk>

.SUFFIXES       : .po

.c.po           : 
	$(CC) $(CFLAGS) -pg -c $(.IMPSRC)

.s.po           : 
	$(AS) $(AFLAGS) -o $(.TARGET) $(.IMPSRC)

$(MACHINES)     : ${.TARGET:S%$%.md/lib$(NAME).a%}
${MACHINES:S%$%.md/lib$(NAME).a%g}      : MAKELIB \
	   $(.TARGET)(${.TARGET:H:R:S/^/\$(/:S%$%OBJS)%}) 

${MACHINES:S%$%.md/lib$(NAME)_p.a%g}    : MAKELIB \
	   $(.TARGET)(${.TARGET:H:R:S/^/\$(/:S%$%OBJS:S,.o\$,.po,g)%}) 

#endif

#
# Harrier. Make the depend<mach> target depend on <mach>.md/dependencies.mk,
# then make that depend on $(<mach>SRCS), but replace .y and .l suffixes with
# .c so we've got a real C file on which we can run our script. In addition,
# the SRCS variable will contain header files, which we don't want, so
# we apply :N*.h: to remove them..
#
$(DEPENDALL)    : ${.TARGET:S%^depend%%:S%$%.md/dependencies.mk%}
${MACHINES:S%$%.md/dependencies.mk%g}   : .EXPORTSAME \
	   ${.TARGET:H:R:S/^/\$(/:S/$/SRCS:S%.y\$%.c%g:S%.l\$%.c%g)/}
	-mv -f $(.TARGET) $(.TARGET)~
	echo "# DO NOT MODIFY THIS FILE; pmake depend will nuke it" >$(.TARGET)
#
#
		

#
# Now the simple targets to run the proper thing for all machines
#
all             : $(MACHINES)
depend          : $(DEPENDALL)

mkmf            ::
	mkmf -f $(MAKEFILE)
